import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import {
  CreateRouteProps,
  Path,
  Role,
  RouteProps,
  PathType,
} from './route.types';
import { WaypointProps } from './value-objects/waypoint.value-object';

export class RouteEntity extends AggregateRoot<RouteProps> {
  protected readonly _id: AggregateID;

  static create = async (create: CreateRouteProps): Promise<RouteEntity> => {
    const props: RouteProps = await create.georouter.routes(
      this.getPaths(create.roles, create.waypoints),
      create.georouterSettings,
    );
    const route = new RouteEntity({ props });
    return route;
  };

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }

  private static getPaths = (
    roles: Role[],
    waypoints: WaypointProps[],
  ): Path[] => {
    const paths: Path[] = [];
    if (roles.includes(Role.DRIVER) && roles.includes(Role.PASSENGER)) {
      if (waypoints.length == 2) {
        // 2 points => same route for driver and passenger
        const commonPath: Path = {
          type: PathType.COMMON,
          points: waypoints,
        };
        paths.push(commonPath);
      } else {
        const driverPath: Path = RouteEntity.createDriverPath(waypoints);
        const passengerPath: Path = RouteEntity.createPassengerPath(waypoints);
        paths.push(driverPath, passengerPath);
      }
    } else if (roles.includes(Role.DRIVER)) {
      const driverPath: Path = RouteEntity.createDriverPath(waypoints);
      paths.push(driverPath);
    } else if (roles.includes(Role.PASSENGER)) {
      const passengerPath: Path = RouteEntity.createPassengerPath(waypoints);
      paths.push(passengerPath);
    }
    return paths;
  };

  private static createDriverPath = (waypoints: WaypointProps[]): Path => {
    return {
      type: PathType.DRIVER,
      points: waypoints,
    };
  };

  private static createPassengerPath = (waypoints: WaypointProps[]): Path => {
    return {
      type: PathType.PASSENGER,
      points: [waypoints[0], waypoints[waypoints.length - 1]],
    };
  };
}

// import { IGeodesic } from '../interfaces/geodesic.interface';
// import { Point } from '../types/point.type';
// import { SpacetimePoint } from './spacetime-point';

// export class Route {
//   distance: number;
//   duration: number;
//   fwdAzimuth: number;
//   backAzimuth: number;
//   distanceAzimuth: number;
//   points: Point[];
//   spacetimePoints: SpacetimePoint[];
//   private geodesic: IGeodesic;

//   constructor(geodesic: IGeodesic) {
//     this.distance = undefined;
//     this.duration = undefined;
//     this.fwdAzimuth = undefined;
//     this.backAzimuth = undefined;
//     this.distanceAzimuth = undefined;
//     this.points = [];
//     this.spacetimePoints = [];
//     this.geodesic = geodesic;
//   }

//   setPoints = (points: Point[]): void => {
//     this.points = points;
//     this.setAzimuth(points);
//   };

//   setSpacetimePoints = (spacetimePoints: SpacetimePoint[]): void => {
//     this.spacetimePoints = spacetimePoints;
//   };

//   protected setAzimuth = (points: Point[]): void => {
//     const inverse = this.geodesic.inverse(
//       points[0].lon,
//       points[0].lat,
//       points[points.length - 1].lon,
//       points[points.length - 1].lat,
//     );
//     this.fwdAzimuth =
//       inverse.azimuth >= 0 ? inverse.azimuth : 360 - Math.abs(inverse.azimuth);
//     this.backAzimuth =
//       this.fwdAzimuth > 180 ? this.fwdAzimuth - 180 : this.fwdAzimuth + 180;
//     this.distanceAzimuth = inverse.distance;
//   };
// }
